package cc.fascinated.bat.features.birthday.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.birthday.profile.BirthdayProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;


/**
 * @author Fascinated (fascinated7)
 */
@Component("birthday:message.sub")
@CommandInfo(name = "message", description = "Changes the message that is sent when it is a user's birthday", requiredPermissions = Permission.MANAGE_SERVER)
public class MessageSubCommand extends BatCommand {
    @Autowired
    public MessageSubCommand() {
        super.addOptions(new OptionData(OptionType.STRING, "message", "The message that is sent. (Placeholders: {user}, {age})", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        BirthdayProfile profile = guild.getBirthdayProfile();
        OptionMapping messageOption = event.getOption("message");
        if (messageOption == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You must provide a message")
                    .build()).queue();
            return;
        }

        String message = messageOption.getAsString();
        if (message.length() > 2000) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The message must be less than 2000 characters")
                    .build()).queue();
            return;
        }
        if (!message.contains("{user}") || !message.contains("{age}")) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The message must contain the placeholders {user} and {age}")
                    .build()).queue();
            return;
        }

        profile.setMessage(message);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("You have updated the birthday message!\n\n**Message:** %s".formatted(profile.getBirthdayMessage(user.getDiscordUser())))
                .build()).queue();
    }
}
