package cc.fascinated.bat.features.minecraft.command.serverwatcher;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.minecraft.MinecraftProfile;
import cc.fascinated.bat.features.minecraft.ServerWatcher;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.entities.channel.unions.GuildChannelUnion;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;
import xyz.mcutils.McUtilsAPI;
import xyz.mcutils.models.server.MinecraftServer;
import xyz.mcutils.models.server.ServerPlatform;

/**
 * @author Fascinated (fascinated7)
 */
@Component("minecraft-server-watcher.add:sub")
@CommandInfo(
        name = "add",
        description = "Add a server to the server watcher"
)
public class AddSubCommand extends BatCommand {
    public AddSubCommand() {
        super.addOptions(
                new OptionData(OptionType.CHANNEL, "channel", "The channel to send the server watcher notifications", true),
                new OptionData(OptionType.STRING, "platform", "The platform of the server to lookup", true)
                        .addChoice("Java", "java")
                        .addChoice("Bedrock", "bedrock"),
                new OptionData(OptionType.STRING, "host", "The host/ip of the server to add", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping platformOption = event.getOption("platform");
        OptionMapping hostOption = event.getOption("host");
        OptionMapping channelOption = event.getOption("channel");
        if (platformOption == null || hostOption == null || channelOption == null) {
            return;
        }
        String platform = platformOption.getAsString();
        String host = hostOption.getAsString();
        GuildChannelUnion channelUnion = channelOption.getAsChannel();
        TextChannel textChannel = channelUnion.asTextChannel();

        MinecraftServer server;
        try {
            if (platform.equalsIgnoreCase("java")) {
                server = McUtilsAPI.getJavaServer(host);
            } else {
                server = McUtilsAPI.getBedrockServer(host);
            }
        } catch (Exception ex) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The server `%s` is invalid or offline".formatted(host))
                    .build()).queue();
            return;
        }

        MinecraftProfile profile = guild.getMinecraftProfile();
        if (profile.getServerWatchers().size() >= 10) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You can only have a maximum of `10` server watchers")
                    .build()).queue();
            return;
        }

        if (profile.getServerWatcher(host, ServerPlatform.valueOf(platform.toUpperCase())) != null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The server `%s` is already being watched".formatted(host))
                    .build()).queue();
            return;
        }

        profile.addServerWatcher(new ServerWatcher(
                server.getHostname(),
                server.getPort(),
                ServerPlatform.valueOf(platform.toUpperCase()),
                textChannel.getId(),
                false
        ));
        profile.checkServers(); // Force check the servers

        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Setup the server watcher for `%s` in %s".formatted(
                        server.getHostname(),
                        textChannel.getAsMention()
                )).build()).queue();
    }
}
