package cc.fascinated.bat.features.minecraft.command.serverwatcher;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.EnumUtils;
import cc.fascinated.bat.features.minecraft.MinecraftProfile;
import cc.fascinated.bat.features.minecraft.ServerWatcher;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.apache.catalina.Server;
import org.springframework.stereotype.Component;
import xyz.mcutils.models.server.ServerPlatform;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Component("minecraft-server-watcher.list:sub")
@CommandInfo(
        name = "list",
        description = "Shows a list of all the servers being watched"
)
public class ListSubCommand extends BatCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        MinecraftProfile profile = guild.getMinecraftProfile();

        Map<ServerPlatform, List<ServerWatcher>> watchers = new HashMap<>();
        for (ServerWatcher server : profile.getServerWatchers()) {
            watchers.computeIfAbsent(server.getPlatform(), k -> new ArrayList<>()).add(server);
        }
        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Server Watcher");
        description.appendLine("Here is a list of all the servers being watched", false);
        description.emptyLine();

        for (Map.Entry<ServerPlatform, List<ServerWatcher>> entry : watchers.entrySet()) {
            description.appendLine("**%s**".formatted(EnumUtils.getEnumName(entry.getKey())), false);
            for (ServerWatcher server : entry.getValue()) {
                description.appendLine("`%s` - %s".formatted(
                        server.getHostname(),
                        server.getChannel().getAsMention()
                ), true);
            }
            description.emptyLine();
        }

        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription(description.build())
                .build()).queue();
    }
}
