package cc.fascinated.bat.features.spotify.profile;

import cc.fascinated.bat.common.Serializable;
import com.google.gson.Gson;
import lombok.Getter;
import lombok.Setter;
import org.bson.Document;

/**
 * @author Fascinated (fascinated7)
 */
@Getter @Setter
public class SpotifyProfile extends Serializable {
    /**
     * The access token
     */
    private String accessToken;

    /**
     * The refresh token
     */
    private String refreshToken;

    /**
     * When the access token expires
     */
    private Long expiresAt;

    /**
     * Checks if the account has a linked account
     *
     * @return if the account has a linked account
     */
    public boolean hasLinkedAccount() {
        return this.accessToken != null && this.refreshToken != null;
    }

    @Override
    public void reset() {
        this.accessToken = null;
        this.refreshToken = null;
    }

    @Override
    public void load(Document document, Gson gson) {
        this.accessToken = document.getString("accessToken");
        this.refreshToken = document.getString("refreshToken");
        this.expiresAt = document.getLong("expiresAt");
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        document.put("accessToken", this.accessToken);
        document.put("refreshToken", this.refreshToken);
        document.put("expiresAt", this.expiresAt);
        return document;
    }
}
